@@ -3,6 +3,6 @@ import qrogue.game
 import qrogue.graphics
 import qrogue.management
 
-from qrogue.qrogue import start_game
+from qrogue.qrogue import start_qrogue
 from qrogue.qrogue import setup_game
 from qrogue.qrogue import validate_map
