@@ -1,5 +1,5 @@
 # exporting
-from .state_vector import StateVector
+from .state_vector import StateVector, CircuitMatrix
 from .controllables import Controllable, Player, Robot
 from .controllables import robot
 from .puzzles import Enemy, Boss, Riddle
