@@ -1,8 +1,9 @@
 # exporting
 from .target import Target
-from .enemy import Enemy, DummyEnemy
+from .enemy import Enemy
 from .boss import Boss
 from .riddle import Riddle
+from .challenge import Challenge
 
 # importing
 # +util
