@@ -1,4 +1,4 @@
-from typing import Tuple
+from typing import Tuple, Optional
 
 from qrogue.game.logic.actors import StateVector
 from qrogue.game.logic.collectibles import Collectible
@@ -19,9 +19,15 @@ class Riddle(Target):
     def is_active(self) -> bool:
         return super(Riddle, self).is_active and self.attempts > 0
 
-    def is_reached(self, state_vector: StateVector) -> Tuple[bool, Collectible]:
+    @property
+    def flee_energy(self) -> int:
+        return 0    # fleeing/aborting/pausing riddles is for free
+
+    def is_reached(self, state_vector: StateVector) -> Tuple[bool, Optional[Collectible]]:
         success, reward = super(Riddle, self).is_reached(state_vector)
-        if not success:
+        if success:
+            assert reward is not None  # riddles always need to give a reward
+        else:
             self.__attempts -= 1
         return success, reward
 
