@@ -47,10 +47,12 @@ class Key(Pickup):
         return "Key"
 
     def description(self) -> str:
-        return "A Key is useful for opening locked doors."
+        return "Keys are useful for opening locked doors."
 
     def to_string(self):
-        return f"{self.amount} key(s)"
+        if self.amount > 1:
+            return f"{self.amount} keys"
+        return f"{self.amount} key"
 
     def __str__(self) -> str:
         return self.to_string()
@@ -64,10 +66,13 @@ class Energy(Pickup):
         return "Energy"
 
     def description(self) -> str:
-        return "Gives back some energy to the robot so it can stay longer in a dungeon."
+        return "Gives back some energy to the Robot so it can stay longer on a mission."
 
     def to_string(self) -> str:
         return f"Energy ({self.amount})"
 
+    def default_price(self) -> int:
+        return 2 + int(self.amount / 7)
+
     def __str__(self) -> str:
         return self.to_string()
\ No newline at end of file
