@@ -1,6 +1,6 @@
 # exporting
 from .rooms import Area, Hallway, Room, BossRoom, MetaRoom, RiddleRoom, ShopRoom, SpawnRoom, TreasureRoom, WildRoom
-from .map import MapType, Map
+from .map import MapType, MapMetaData, Map
 from .spaceship_map import SpaceshipMap
 from .world_map import WorldMap
 from .level_map import LevelMap
