@@ -1,15 +1,15 @@
 import math
 from abc import ABC, abstractmethod
 from enum import Enum
-from typing import Dict, List, Callable
+from typing import Dict, List, Callable, Optional
 
+from qrogue.game import target_factory
 from qrogue.game.logic import Message
 from qrogue.game.logic.actors import Robot, Riddle
-from qrogue.game import target_factory
 from qrogue.game.logic.collectibles import Instruction
 from qrogue.game.world.navigation import Coordinate, Direction
-from qrogue.game.world.tiles import Enemy as EnemyTile, Tile, Floor, Decoration, Teleport, FogOfWar, Void, Invalid, Door, \
-    Wall, HallwayEntrance, Riddler, ShopKeeper, Boss, Collectible, Message as MessageTile
+from qrogue.game.world.tiles import Enemy as EnemyTile, Tile, Floor, Decoration, Teleport, FogOfWar, Void, Invalid, \
+    Door, Wall, HallwayEntrance, Riddler, ShopKeeper, Boss, Collectible, Message as MessageTile
 from qrogue.util import CommonQuestions, MapConfig, Logger, CheatConfig, RandomManager
 
 
@@ -33,20 +33,20 @@ class Area(ABC):
     __ID = 1
     __FOG = FogOfWar()
     __VOID = Void()
-    UNIT_WIDTH = 7
-    UNIT_HEIGHT = 7
-    MID_X = int(UNIT_WIDTH / 2)
-    MID_Y = int(UNIT_HEIGHT / 2)
+    UNIT_WIDTH = MapConfig.room_width()     # todo change visibility of UNIT_WIDTH?
+    UNIT_HEIGHT = MapConfig.room_height()
+    MID_X = MapConfig.room_mid_x()
+    MID_Y = MapConfig.room_mid_y()
 
     @staticmethod
     def void() -> Void:
         return Area.__VOID
 
-    def __init__(self, type: AreaType, tile_matrix: "list[list[Tile]]"):
+    def __init__(self, type_: AreaType, tile_matrix: "list[list[Tile]]"):
         self.__id = Area.__ID
         Area.__ID += 1
 
-        self.__type = type
+        self.__type = type_
         self.__tiles = tile_matrix
         self.__width = len(tile_matrix[0])
         self.__height = len(tile_matrix)
@@ -60,7 +60,7 @@ class Area(ABC):
         return self.__id
 
     @property
-    def _is_visible(self) -> bool:
+    def is_visible(self) -> bool:
         return self.__is_visible or CheatConfig.revealed_map()
 
     @property
@@ -68,7 +68,7 @@ class Area(ABC):
         return self.__is_visible
 
     @property
-    def _is_in_sight(self) -> bool:
+    def is_in_sight(self) -> bool:
         return self.__is_in_sight
 
     @property
@@ -97,10 +97,10 @@ class Area(ABC):
         :return: the Tile at the requested position
         """
         if 0 <= x < self.__width and 0 <= y < self.__height:
-            if self._is_visible or force:
+            if self.is_visible or force:
                 return self.__tiles[y][x]
             else:
-                if self._is_in_sight:
+                if self.is_in_sight:
                     return Area.__FOG
                 else:
                     return Area.void()
@@ -111,10 +111,10 @@ class Area(ABC):
         if row >= len(self.__tiles):
             return "".join([Invalid().get_img()] * Area.UNIT_WIDTH)
 
-        if self._is_visible:
+        if self.is_visible:
             tiles = [t.get_img() for t in self.__tiles[row]]
             return "".join(tiles)
-        elif self._is_in_sight:
+        elif self.is_in_sight:
             fog_str = Area.__FOG.get_img()
             return fog_str * self.__width
         else:
@@ -187,9 +187,8 @@ class Placeholder:
         return Placeholder.__ROOM
 
     @staticmethod
-    def empty_room(north_hallway: "Hallway" = None, east_hallway: "Hallway" = None, south_hallway: "Hallway" = None,
-                   west_hallway: "Hallway" = None) -> "Room":
-        return EmptyRoom(north_hallway, east_hallway, south_hallway, west_hallway)
+    def empty_room(hw_dic: Dict[Direction, "Hallway"]) -> "CopyAbleRoom":
+        return EmptyRoom(hw_dic)
 
 
 class Hallway(Area):
@@ -252,16 +251,19 @@ class Hallway(Area):
         if self.__room1:
             self.__room1.in_sight()
         else:
-            Logger.instance().debug("room1 is None!")
+            Logger.instance().debug("room1 is None!", from_pycui=False)
         if self.__room2:
             self.__room2.in_sight()
         else:
-            Logger.instance().debug("room2 is None!")
+            Logger.instance().debug("room2 is None!", from_pycui=False)
 
     def get_row_str(self, row: int) -> str:
         if self.__hide:
             if self.__door.check_event():
-                self.make_visible()
+                if self.__room1.is_visible or self.__room2.is_visible:
+                    self.make_visible()
+                # elif self.__room1.in_sight or self.__room2.in_sight:
+                #    self.in_sight()
                 self.__hide = False
         return super(Hallway, self).get_row_str(row)
 
@@ -322,7 +324,7 @@ class Room(Area):
                                                        f"{len(tile_list)}!"))
         return tile_list
 
-    def __init__(self, type: AreaType, tile_list: List[Tile], north_hallway: Hallway = None,
+    def __init__(self, type_: AreaType, tile_list: List[Tile], north_hallway: Hallway = None,
                  east_hallway: Hallway = None, south_hallway: Hallway = None, west_hallway: Hallway = None):
         tiles = []
         room_top = [Wall()] * Area.UNIT_WIDTH
@@ -365,7 +367,7 @@ class Room(Area):
             self.__hallways[Direction.West] = west_hallway
             west_hallway.set_room(self, Direction.West)
 
-        super(Room, self).__init__(type, tiles)
+        super(Room, self).__init__(type_, tiles)
 
     def _set_hallway(self, north_hallway: Hallway = None, east_hallway: Hallway = None, south_hallway: Hallway = None,
                      west_hallway: Hallway = None):
@@ -389,12 +391,12 @@ class Room(Area):
             self.__hallways[Direction.West] = west_hallway
             west_hallway.set_room(self, Direction.West)
 
-    def get_hallway(self, direction: Direction, throw_error: bool = True) -> Hallway:
+    def get_hallway(self, direction: Direction, throw_error: bool = True) -> Optional[Hallway]:
         if direction in self.__hallways:
             return self.__hallways[direction]
         elif throw_error:
             dic = [(str(self.__hallways[k]) + "\n") for k in self.__hallways]
-            Logger.instance().error(f"Invalid hallway access for {self}:\n{direction} not in {dic}")
+            Logger.instance().error(f"Invalid hallway access for {self}:\n{direction} not in {dic}", from_pycui=False)
         return None
 
     def make_visible(self):
@@ -480,9 +482,9 @@ class MetaRoom(CopyAbleRoom):
             elif orientation is Direction.West:
                 self._set(tile_list, 0, 0, type_decoration)
                 self._set(tile_list, 0, Room.INNER_MID_Y, dash_decoration)
-                self._set(tile_list,0, Room.INNER_HEIGHT - 1, num_decoration)
+                self._set(tile_list, 0, Room.INNER_HEIGHT - 1, num_decoration)
                 self._set(tile_list, Room.INNER_WIDTH - 1, 0, msg_tile)
-                self._set(tile_list, Room.INNER_WIDTH - 1, Room.INNER_HEIGHT - 1, msg_tile)
+                self._set(tile_list, Room.INNER_WIDTH - 1, Room.INNER_HEIGHT - 1, level_tile)
                 coordinate = Coordinate(Room.INNER_MID_X - 1, 0)
                 addition = Direction.South
 
@@ -515,19 +517,19 @@ class MetaRoom(CopyAbleRoom):
         if self._is_visible_value:
             # don't use Room's implementation but Area's
             super(CopyAbleRoom, new_room).make_visible()
-        elif self._is_in_sight:
+        elif self.is_in_sight:
             new_room.in_sight()
         return new_room
 
 
 class CustomRoom(CopyAbleRoom):
-    def __init__(self, type: AreaType, tile_matrix: List[List[Tile]], north_hallway: Hallway = None,
+    def __init__(self, type_: AreaType, tile_matrix: Optional[List[List[Tile]]], north_hallway: Hallway = None,
                  east_hallway: Hallway = None, south_hallway: Hallway = None, west_hallway: Hallway = None):
         self.__tile_matrix = tile_matrix
         tile_list = []
         for row in tile_matrix:
             tile_list += row
-        super().__init__(type, tile_list, north_hallway, east_hallway, south_hallway, west_hallway)
+        super().__init__(type_, tile_list, north_hallway, east_hallway, south_hallway, west_hallway)
 
     def abbreviation(self) -> str:
         return "CR"
@@ -540,19 +542,35 @@ class CustomRoom(CopyAbleRoom):
             new_room = CustomRoom(self.type, self.__tile_matrix, hw_dic[Direction.North], hw_dic[Direction.East],
                                   hw_dic[Direction.South], hw_dic[Direction.West])
         else:
-            new_room = CustomRoom(self.type, self.__tile_matrix)   # todo not sure if rooms without hallways should be legal
+            # todo not sure if rooms without hallways should be legal
+            new_room = CustomRoom(self.type, self.__tile_matrix)
 
         if self._is_visible_value:
             # don't use Room's implementation but Area's
             super(Room, new_room).make_visible()
-        elif self._is_in_sight:
+        elif self.is_in_sight:
             new_room.in_sight()
         return new_room
 
 
-class EmptyRoom(Room):
-    def __init__(self, north_hallway: Hallway = None, east_hallway: Hallway = None, south_hallway: Hallway = None,
-                 west_hallway: Hallway = None):
+class EmptyRoom(CopyAbleRoom):
+    def __init__(self, hw_dic: Dict[Direction, Hallway]):
+        north_hallway = None
+        if Direction.North in hw_dic:
+            north_hallway = hw_dic[Direction.North]
+
+        east_hallway = None
+        if Direction.East in hw_dic:
+            east_hallway = hw_dic[Direction.East]
+
+        south_hallway = None
+        if Direction.South in hw_dic:
+            south_hallway = hw_dic[Direction.South]
+
+        west_hallway = None
+        if Direction.West in hw_dic:
+            west_hallway = hw_dic[Direction.West]
+
         tile_list = Room.get_empty_room_tile_list()
         super(EmptyRoom, self).__init__(AreaType.EmptyRoom, tile_list, north_hallway, east_hallway, south_hallway,
                                         west_hallway)
@@ -560,9 +578,12 @@ class EmptyRoom(Room):
     def abbreviation(self) -> str:
         return "ER"
 
+    def copy(self, hw_dic: Dict[Direction, Hallway]) -> "CopyAbleRoom":
+        return EmptyRoom(hw_dic)
+
 
 class SpecialRoom(Room, ABC):
-    def __init__(self, type: AreaType, hallway: Hallway, direction: Direction, tile_dic: Dict[Coordinate, Tile]):
+    def __init__(self, type_: AreaType, hallway: Hallway, direction: Direction, tile_dic: Dict[Coordinate, Tile]):
         """
 
         :param hallway:
@@ -572,28 +593,30 @@ class SpecialRoom(Room, ABC):
         tile_list = Room.dic_to_tile_list(tile_dic)
         if Hallway.is_first(direction):
             if hallway.connects_horizontally():
-                super(SpecialRoom, self).__init__(type, tile_list, east_hallway=hallway)
+                super(SpecialRoom, self).__init__(type_, tile_list, east_hallway=hallway)
             else:
-                super(SpecialRoom, self).__init__(type, tile_list, south_hallway=hallway)
+                super(SpecialRoom, self).__init__(type_, tile_list, south_hallway=hallway)
         else:
             if hallway.connects_horizontally():
-                super(SpecialRoom, self).__init__(type, tile_list, west_hallway=hallway)
+                super(SpecialRoom, self).__init__(type_, tile_list, west_hallway=hallway)
             else:
-                super(SpecialRoom, self).__init__(type, tile_list, north_hallway=hallway)
+                super(SpecialRoom, self).__init__(type_, tile_list, north_hallway=hallway)
 
 
-class SpawnRoom(Room):
-    def __init__(self, load_map_callback: Callable[[str, Coordinate], None], tile_dic: Dict[Coordinate, Tile] = None,
-                 north_hallway: Hallway = None, east_hallway: Hallway = None, south_hallway: Hallway = None,
-                 west_hallway: Hallway = None):
+class SpawnRoom(CopyAbleRoom):
+    def __init__(self, load_map_callback: Callable[[str, Optional[Coordinate]], None],
+                 tile_dic: Dict[Coordinate, Tile] = None, north_hallway: Hallway = None, east_hallway: Hallway = None,
+                 south_hallway: Hallway = None, west_hallway: Hallway = None, place_teleporter: bool = True):
         room_mid = Coordinate(Room.INNER_MID_X, Room.INNER_MID_Y)
         if tile_dic:
-            if room_mid in tile_dic:
+            if room_mid in tile_dic and place_teleporter:
                 Logger.instance().error("Specified tile_dic with non-empty room-center for SpawnRoom. Overriding it "
-                                        "with Teleporter.", show=False)
+                                        "with Teleporter.", show=False, from_pycui=False)
         else:
             tile_dic = {}
-        tile_dic[room_mid] = Teleport(self.__teleport_callback, MapConfig.back_map_string(), None)
+        if place_teleporter:
+            tile_dic[room_mid] = Teleport(self.__teleport_callback, MapConfig.back_map_string(), None)
+        self.__tile_dic = tile_dic
         tile_list = Room.dic_to_tile_list(tile_dic)
         super().__init__(AreaType.SpawnRoom, tile_list, north_hallway, east_hallway, south_hallway, west_hallway)
         self.__load_map = load_map_callback
@@ -605,32 +628,28 @@ class SpawnRoom(Room):
     def set_is_done_callback(self, is_done_callback: Callable[[], bool]):
         self.__is_done = is_done_callback
 
-    def __teleport_callback(self, level_to_load: str, room: Coordinate):
+    def __teleport_callback(self, level_to_load: str, room: Optional[Coordinate]):
         if self.__is_done:
             if self.__is_done():
                 self.__conditional_going_back(True)
             else:
                 CommonQuestions.GoingBack.ask(self.__conditional_going_back)
         else:
-            Logger.instance().error("is_done_callback not set yet!")
+            Logger.instance().error("is_done_callback not set yet!", from_pycui=False)
 
     def __conditional_going_back(self, confirmed: bool):
         if confirmed:
             self.__load_map(MapConfig.back_map_string(), None)
 
+    def copy(self, hw_dic: Dict[Direction, Hallway]) -> "CopyAbleRoom":
+        # don't place a teleporter because it's already in tile_dic if it should be placed
+        return SpawnRoom(self.__load_map, self.__tile_dic, hw_dic[Direction.North], hw_dic[Direction.East],
+                         hw_dic[Direction.South], hw_dic[Direction.West], place_teleporter=False)
+
 
 class BaseWildRoom(Room):
-    def __init__(self, tile_list: [Tile], get_tiles_by_id: "(int,)", north_hallway: Hallway = None,
+    def __init__(self, tile_list: [Tile], get_tiles_by_id: Callable[[int], List[Tile]], north_hallway: Hallway = None,
                  east_hallway: Hallway = None, south_hallway: Hallway = None, west_hallway: Hallway = None):
-        """
-
-        :param factory:
-        :param chance: chance for every individual Tile of the room to be an EnemyTile
-        :param east_door: the Door connecting to the Room to the East of this one
-        :param south_door: the Door connecting to the Room to the South of this one
-        :param west_door: whether the Room to the West is having an east_door or not
-        :param north_door:  whether the Room to the North is having a south_door or not
-        """
         self.get_tiles_by_id = get_tiles_by_id
         super().__init__(AreaType.WildRoom, tile_list, north_hallway, east_hallway, south_hallway, west_hallway)
 
@@ -643,23 +662,14 @@ class WildRoom(BaseWildRoom):
 
     def __init__(self, factory: target_factory.EnemyFactory, chance: float = 0.6, north_hallway: Hallway = None,
                  east_hallway: Hallway = None, south_hallway: Hallway = None, west_hallway: Hallway = None):
-        """
-
-        :param factory:
-        :param chance: chance for every individual Tile of the room to be an EnemyTile
-        :param east_door: the Door connecting to the Room to the East of this one
-        :param south_door: the Door connecting to the Room to the South of this one
-        :param west_door: whether the Room to the West is having an east_door or not
-        :param north_door:  whether the Room to the North is having a south_door or not
-        """
-        self.__dictionary = { 1: [], 2: [], 3: [], 4: [], 5: [], 6: [], 7: [], 8: [], 9: [] }
+        self.__dictionary = {1: [], 2: [], 3: [], 4: [], 5: [], 6: [], 7: [], 8: [], 9: []}
         rm = RandomManager.create_new()
 
         available_positions = []
         for y in range(Room.INNER_HEIGHT):
             available_positions += [Coordinate(x, y) for x in range(Room.INNER_WIDTH)]
         num_of_enemies = len(available_positions) * chance
-        if rm.get() < 0.5:
+        if rm.get(msg="WR_enemyPlacementRounding") < 0.5:
             num_of_enemies = math.floor(num_of_enemies)
         else:
             num_of_enemies = math.ceil(num_of_enemies)
@@ -668,17 +678,20 @@ class WildRoom(BaseWildRoom):
 
         tile_list = Room.get_empty_room_tile_list()
         for i in range(num_of_enemies):
-            id = rm.get_int(min=0, max=WildRoom.__NUM_OF_ENEMY_GROUPS + 1)
-            enemy = EnemyTile(factory, self.__get_tiles_by_id, id)
-            if id > 0:
-                self.__dictionary[id].append(enemy)
-            pos = rm.get_element(available_positions, remove=True)
+            eid = rm.get_int(min_=0, max_=WildRoom.__NUM_OF_ENEMY_GROUPS + 1, msg="WildRoom_eid")
+            enemy = EnemyTile(factory, self.__get_tiles_by_id, self.__update_entangled_tiles, eid)
+            if eid > 0:
+                self.__dictionary[eid].append(enemy)
+            pos = rm.get_element(available_positions, remove=True, msg="WildRoom_epos")
             tile_list[Room.coordinate_to_index(pos)] = enemy
 
         super().__init__(tile_list, self.__get_tiles_by_id, north_hallway, east_hallway, south_hallway, west_hallway)
 
-    def __get_tiles_by_id(self, id: int) -> List[EnemyTile]:
-        return self.__dictionary[id]
+    def __get_tiles_by_id(self, eid: int) -> List[EnemyTile]:
+        return self.__dictionary[eid]
+
+    def __update_entangled_tiles(self, enemy: EnemyTile):
+        self.__dictionary[enemy.eid].append(enemy)
 
 
 class DefinedWildRoom(BaseWildRoom):
@@ -691,8 +704,8 @@ class DefinedWildRoom(BaseWildRoom):
         super().__init__(tile_list, self.__get_tiles_by_id, north_hallway=north_hallway, east_hallway=east_hallway,
                          south_hallway=south_hallway, west_hallway=west_hallway)
 
-    def __get_tiles_by_id(self, id: int) -> List[EnemyTile]:
-        return self.__dictionary[id]
+    def __get_tiles_by_id(self, id_: int) -> List[EnemyTile]:
+        return self.__dictionary[id_]
 
 
 class TreasureRoom(SpecialRoom):
@@ -732,7 +745,7 @@ class ShopRoom(SpecialRoom):
 
 
 class BossRoom(SpecialRoom):
-    def __init__(self, hallway: Hallway, direction: Direction, boss: Boss, tile_dic: "dic of Coordinate and Tile" = None):
+    def __init__(self, hallway: Hallway, direction: Direction, boss: Boss, tile_dic: Dict[Coordinate, Tile] = None):
         super().__init__(AreaType.BossRoom, hallway, direction, tile_dic)
         self._set_tile(boss, x=Area.MID_X, y=Area.MID_Y)
 
