@@ -1,16 +1,18 @@
 import random
-from typing import Callable, List
+from typing import Callable, List, Optional
 
 from qrogue.game.logic.actors import Robot, Controllable, Player
 from qrogue.game.world.navigation import Direction, Coordinate
 from qrogue.game.world.tiles import Tile, TileCode, WalkTriggerTile
-from qrogue.util import MyRandom, Config
+from qrogue.game.world.tiles.tiles import NpcTile
+from qrogue.util import Config, achievements
 
+SCIENTIST_TILE_REPRESENTATION = Config.scientist_name()[0]
 ascii_spaceship = \
     r"                                                                                              " + "\n" \
     r"                                    X---------------X                                         " + "\n" \
     r"                                   /ööööööööööööööööö)>                                       " + "\n" \
-    r"                                  /öööööööööööööööööö)>                           /)          " + "\n" \
+    r"                                  /öTöööööBöööööööööö)>                           /)          " + "\n" \
     r"                                 /ööööööööööööööööööö)>                          /ö|          " + "\n" \
     r"                                |öööööööööööööööööööö)>                         /öö|          " + "\n" \
     r"                                |öööööööööööööööööööö)>                        /ööö|          " + "\n" \
@@ -23,11 +25,11 @@ ascii_spaceship = \
     r"            /ööööööööööööööööööööööööööööööööööööö\                     /öööööööööö|          " + "\n" \
     r"           /ööööööööWöööööööööööööööööööööööööööööö--------------------Xööööööööööö|          " + "\n" \
     r"          |öööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööö|          " + "\n" \
-    r"          |ööSööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööö|          " + "\n" \
+    r"          |ööNööööööööööRöööööööööööööööööööööööööööööööööööööööööööööööööööööööööö|          " + "\n" \
     r"          |öööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööööö|          " + "\n" \
     r"           \ööööööööööööööööööööööööööööööööööööööö--------------------Xööööööööööö|          " + "\n" \
     r"            \ööööööööööööööööööööööööööööööööööööö/                     \öööööööööö|          " + "\n" \
-    r"             \ööööööööööölööööööööööööööööööööööö/                       \ööööööööö|          " + "\n" \
+    r"             \öööööööööööQööööööööööööööööööööööö/                       \ööööööööö|          " + "\n" \
     r"              X-------------Xööööööööööööööööööö(                         \öööööööö|          " + "\n" \
     r"                             \ööööööööööööööööööö\                         \ööööööö|          " + "\n" \
     r"                              \ööööööööööööööööööö\                         \öööööö|          " + "\n" \
@@ -44,7 +46,7 @@ ascii_spaceship = \
 
 
 __rm = random.Random()
-def spaceship_random() -> float:
+def _spaceship_random() -> float:
     return __rm.random()
 
 
@@ -61,6 +63,9 @@ class SpaceshipWallTile(Tile):
     def is_walkable(self, direction: Direction, robot: Robot) -> bool:
         return False
 
+    def copy(self) -> "Tile":
+        return SpaceshipWallTile(self.__img)
+
     def __str__(self):
         return "W"
 
@@ -78,15 +83,20 @@ class SpaceshipFreeWalkTile(Tile):
     def is_walkable(self, direction: Direction, robot: Robot) -> bool:
         return True
 
+    def copy(self) -> "Tile":
+        return SpaceshipFreeWalkTile()
+
     def __str__(self):
         return "f"
 
 
 class SpaceshipTriggerTile(WalkTriggerTile):
-    MAP_START_REPRESENTATION = "S"
+    BED_SPAWN_REPRESENTATION = "B"
+    MAP_START_REPRESENTATION = "N"
     MAP_WORKBENCH_REPRESENTATION = "W"
     MAP_GATE_LIBRARY_REPRESENTATION = "G"
-    START_TEST_LEVEL = "l"
+    QUICKSTART_LEVEL = "Q"      # Quickstart
+    TRAININGS_ROOM = "T"
 
     def __init__(self, character: str, callback: Callable[[Direction, Robot], None]):
         super().__init__(TileCode.SpaceshipTrigger)
@@ -99,6 +109,9 @@ class SpaceshipTriggerTile(WalkTriggerTile):
     def get_img(self):
         return self.__img
 
+    def _copy(self) -> "Tile":
+        return SpaceshipTriggerTile(self.__img, self.__callback)
+
 
 class OuterSpaceTile(Tile):
     MAP_REPRESENTATION = " "
@@ -107,21 +120,24 @@ class OuterSpaceTile(Tile):
 
     def __init__(self):
         super(OuterSpaceTile, self).__init__(TileCode.OuterSpace)
-        self.__is_star = spaceship_random() < 0.007
+        self.__is_star = _spaceship_random() < 0.007
 
     def get_img(self):
         if self.__is_star:
-            if spaceship_random() < OuterSpaceTile.__STAR_DIE_CHANCE:
+            if _spaceship_random() < OuterSpaceTile.__STAR_DIE_CHANCE:
                 self.__is_star = False
             return "*"
         else:
-            if spaceship_random() < OuterSpaceTile.__STAR_BIRTH_CHANCE:
+            if _spaceship_random() < OuterSpaceTile.__STAR_BIRTH_CHANCE:
                 self.__is_star = True
             return self._invisible
 
     def is_walkable(self, direction: Direction, robot: Robot) -> bool:
         return False
 
+    def copy(self) -> "Tile":
+        return OuterSpaceTile()
+
     def __str__(self):
         return "o"
 
@@ -131,14 +147,21 @@ class SpaceshipMap:
     HEIGHT = ascii_spaceship.count("\n")
     SPAWN_POS = Coordinate(x=25, y=16)
 
-    def __init__(self, seed: int, player: Player, open_world_view: Callable[[Direction, Controllable], None],
+    def __init__(self, player: Player, scientist: NpcTile, check_achievement: Callable[[str], bool],
+                 stop_playing: Callable[[Direction, Controllable], None],
+                 open_world_view: Callable[[Direction, Controllable], None],
                  use_workbench: Callable[[Direction, Controllable], None],
-                 start_test_level: Callable[[Direction, Controllable], None]):
-        self.__rm = MyRandom(seed)
+                 load_map: Callable[[str, Optional[Coordinate]], None], load_newest_map: Callable[[], None],
+                 start_training: Callable[[Direction], None]):
         self.__player = player
+        self.__scientist = scientist
+        self.__check_achievement = check_achievement
+        self.__stop_playing_callback = stop_playing
         self.__open_world_view = open_world_view
-        self.__use_workbench = use_workbench
-        self.__start_test_level = start_test_level
+        self.__use_workbench_callback = use_workbench
+        self.__load_map = load_map
+        self.__load_newest_map = load_newest_map
+        self.__start_training = start_training
         self.__tiles = []
         row = []
         for character in ascii_spaceship:
@@ -146,6 +169,9 @@ class SpaceshipMap:
                 self.__tiles.append(row)
                 row = []
             else:
+                if character == SpaceshipTriggerTile.BED_SPAWN_REPRESENTATION:
+                    # we want to start on top of the bed
+                    SpaceshipMap.SPAWN_POS = Coordinate(len(row), len(self.__tiles))
                 tile = self.__ascii_to_tile(character)
                 row.append(tile)
 
@@ -158,23 +184,28 @@ class SpaceshipMap:
         return self.__player_pos
 
     def __ascii_to_tile(self, character: str) -> Tile:
-        if character == SpaceshipFreeWalkTile.MAP_REPRESENTATION:
+        if character == SCIENTIST_TILE_REPRESENTATION:
+            tile = self.__scientist
+        elif character == SpaceshipFreeWalkTile.MAP_REPRESENTATION:
             tile = SpaceshipFreeWalkTile()
         elif character == OuterSpaceTile.MAP_REPRESENTATION:
             tile = OuterSpaceTile()
         elif character == SpaceshipWallTile.MAP_INVISIBLE_PRESENTATION:
             tile = SpaceshipWallTile(" ")
+        elif character == SpaceshipTriggerTile.BED_SPAWN_REPRESENTATION:
+            tile = SpaceshipTriggerTile(character, self.__stop_playing)
         elif character == SpaceshipTriggerTile.MAP_START_REPRESENTATION:
             tile = SpaceshipTriggerTile(character, self.__open_world_view)
         elif character == SpaceshipTriggerTile.MAP_WORKBENCH_REPRESENTATION:
             tile = SpaceshipTriggerTile(character, self.__use_workbench)
         # elif character == SpaceshipTriggerTile.MAP_GATE_LIBRARY_REPRESENTATION:
         #    tile = SpaceshipTriggerTile(character, self.open_gate_library)
-        elif character == SpaceshipTriggerTile.START_TEST_LEVEL:
-            if Config.debugging():
-                tile = SpaceshipTriggerTile(character, self.__start_test_level)
-            else:
-                tile = SpaceshipFreeWalkTile()
+        elif character == SpaceshipTriggerTile.QUICKSTART_LEVEL:
+            tile = SpaceshipTriggerTile(character, self.__load_newest_map)
+        elif character == SpaceshipTriggerTile.TRAININGS_ROOM:
+            def start_training(direction: Direction, controllable: Controllable):
+                self.__start_training(direction)
+            tile = SpaceshipTriggerTile(character, start_training)
         else:
             tile = SpaceshipWallTile(character)
         return tile
@@ -207,6 +238,14 @@ class SpaceshipMap:
         else:
             return False
 
+    def __stop_playing(self, direction: Direction, controllable: Controllable):
+        if self.__check_achievement(achievements.FinishedTutorial):
+            self.__stop_playing_callback(direction, controllable)
+
+    def __use_workbench(self, direction: Direction, controllable: Controllable):
+        if self.__check_achievement(achievements.UnlockedWorkbench):
+            self.__use_workbench_callback(direction, controllable)
+
     def __trigger_event(self, event_id: str):
         # todo check if we really don't need this in the Spaceship
         pass
