@@ -1,9 +1,10 @@
 # exporting
-from .tiles import TileCode, TileColorer, Tile
+from .tiles import TileCode, Tile
 from .tiles import Invalid, Debug, Void, Floor, Wall, Obstacle, FogOfWar, Decoration, ControllableTile
-from .walk_trigger_tiles import WalkTriggerTile, Trigger, Teleport, Message, Riddler, ShopKeeper, Collectible, Energy
+from .walk_trigger_tiles import WalkTriggerTile, Trigger, Teleport, Tunnel, Message, Riddler, ShopKeeper, Collectible, \
+    Energy, Challenger
 from .puzzle_tiles import Enemy, Boss
-from .door_tiles import DoorOpenState, DoorOneWayState, Door, EntangledDoor, HallwayEntrance
+from .door_tiles import DoorOpenState, DoorOneWayState, DoorEntanglementState, Door, HallwayEntrance
 
 # importing
 # +util
