@@ -1,30 +1,31 @@
 from enum import Enum
-from typing import Callable
+from typing import Callable, List
 
 from qrogue.game.logic.actors import Controllable, Robot, Boss as BossActor
 from qrogue.game.target_factory import EnemyFactory
 from qrogue.game.world.navigation import Direction
-from qrogue.util import RandomManager, Logger, CheatConfig
+from qrogue.util import RandomManager, Logger, CheatConfig, PuzzleConfig
 
-from qrogue.game.world.tiles import TileCode, WalkTriggerTile
+from qrogue.game.world.tiles import Tile, TileCode, WalkTriggerTile
 
 
 class _EnemyState(Enum):
     UNDECIDED = 0
-    FREE = 1
+    # FREE = 1
     FIGHT = 2
     DEAD = 3
     FLED = 4
 
 
 class Enemy(WalkTriggerTile):
-    def __init__(self, factory: EnemyFactory, get_entangled_tiles, e_id: int = 0):
+    def __init__(self, factory: EnemyFactory, get_entangled_tiles: Callable[[int], List["Enemy"]],
+                 update_entangled_groups: Callable[["Enemy"], None], e_id: int = 0):
         super().__init__(TileCode.Enemy)
         self.__factory = factory
         self.__state = _EnemyState.UNDECIDED
         self.__get_entangled_tiles = get_entangled_tiles
+        self.__update_entangled_groups = update_entangled_groups
         self.__id = e_id
-        self.__amplitude = 0.1 * (10 - e_id)        # the lower the id the higher the chance of a fight
         self.__rm = RandomManager.create_new()
         self.__enemy = None
 
@@ -32,10 +33,6 @@ class Enemy(WalkTriggerTile):
     def eid(self) -> int:
         return self.__id
 
-    @property
-    def amplitude(self) -> float:
-        return self.__amplitude
-
     @property
     def _state(self) -> _EnemyState:
         if self.__enemy:
@@ -49,10 +46,10 @@ class Enemy(WalkTriggerTile):
             if controllable.backpack.used_capacity > 0:
                 return super(Enemy, self).is_walkable(direction, controllable)
             else:
-                controllable.damage(1)
+                # noting happens in case the robot doesn't have any gates
                 return False
         else:
-            Logger.instance().error(f"Error! Non-Robot walked over Enemy: {controllable}")
+            Logger.instance().error(f"Error! Non-Robot walked over Enemy: {controllable}", from_pycui=False)
 
     def _on_walk(self, direction: Direction, controllable: Controllable) -> bool:
         if isinstance(controllable, Robot):
@@ -60,19 +57,12 @@ class Enemy(WalkTriggerTile):
             if self._state == _EnemyState.UNDECIDED:
                 if self.__measure():
                     self.__fight(robot, direction)
-                else:
-                    self.__state = _EnemyState.FLED
             elif self._state == _EnemyState.FIGHT:
-                if CheatConfig.is_scared_rabbit():
-                    self.__state = _EnemyState.FLED
-                else:
-                    self.__fight(robot, direction)
-            elif self._state == _EnemyState.FREE:
-                self.__state = _EnemyState.FLED
+                self.__fight(robot, direction)
         return False
 
     def get_img(self):
-        if self._state == _EnemyState.DEAD :
+        if self._state == _EnemyState.DEAD:
             return self._invisible
         elif self._state == _EnemyState.FLED:
             return self._invisible
@@ -90,31 +80,34 @@ class Enemy(WalkTriggerTile):
             ))
 
     def __measure(self):
-        if CheatConfig.is_scared_rabbit():
-            return False
-
+        # since self will also be part of what get_entangled_tiles() returns, we act upon self via entangled_tiles
         if 0 < self.__id <= 9:
             entangled_tiles = self.__get_entangled_tiles(self.__id)
         else:
             entangled_tiles = [self]
 
-        state = _EnemyState.FREE
-        if self.__rm.get() < self.amplitude:
+        # first do the randomness check because otherwise cheating could mess with some overall randomness
+        if self.__rm.get(msg="Target.__measure()") >= PuzzleConfig.calculate_appearance_chance(self.__id) \
+                or CheatConfig.is_scared_rabbit():
+            state = _EnemyState.FLED
+        else:
             state = _EnemyState.FIGHT
         for enemy in entangled_tiles:
-            enemy._set_state(state)
+            enemy._set_state(state)     # this will also set self.__state
 
-        # sometimes when there is no entanglement we have to explicitly set the state of self
-        if self._state == _EnemyState.UNDECIDED:
-            self.__state = state
-
-        return state == _EnemyState.FIGHT
+        return self.__state is _EnemyState.FIGHT
 
     def __fight(self, robot: Robot, direction: Direction):
         if self.__enemy is None:
-            self.__enemy = self.__factory.produce(robot, self.__rm)
+            # the higher the amplitude the easier it should be to flee
+            self.__enemy = self.__factory.produce(robot, self.__rm, self.__id)
         self.__factory.start(robot, self.__enemy, direction)
 
+    def _copy(self) -> "Tile":
+        enemy = Enemy(self.__factory, self.__get_entangled_tiles, self.__update_entangled_groups, self.__id)
+        self.__update_entangled_groups(enemy)
+        return enemy
+
     def __str__(self) -> str:
         return f"E({self.__id}|{self._state})"
 
@@ -139,7 +132,7 @@ class Boss(WalkTriggerTile):
             if self._is_active:
                 self.__on_walk_callback(controllable, self.__boss, direction)
         else:
-            Logger.instance().error(f"Non-Robot walked on Boss! controllable = {controllable}")
+            Logger.instance().error(f"Non-Robot walked on Boss! controllable = {controllable}", from_pycui=False)
         return False
 
     def get_img(self):
@@ -147,3 +140,7 @@ class Boss(WalkTriggerTile):
             return "B"
         else:
             return self._invisible
+
+    def _copy(self) -> "Tile":
+        # Bosses should not be duplicated in a level anyways, so it doesn't matter if we reference the same BossActor
+        return Boss(self.__boss, self.__on_walk_callback)
