@@ -3,7 +3,8 @@ from .renderable import Renderable
 from .my_widgets import MyBaseWidget, Widget
 from .widget_sets import MyWidgetSet
 from .widget_sets import ExploreWidgetSet, NavigationWidgetSet
-from .widget_sets import FightWidgetSet, BossFightWidgetSet, RiddleWidgetSet
+from .widget_sets import ReachTargetWidgetSet, TrainingsWidgetSet, FightWidgetSet, BossFightWidgetSet, \
+    RiddleWidgetSet, ChallengeWidgetSet
 from .widget_sets import MenuWidgetSet, PauseMenuWidgetSet, ShopWidgetSet, WorkbenchWidgetSet
 from .spaceship import SpaceshipWidgetSet
 
