@@ -1,16 +1,15 @@
 
 import random
 import sys
-from typing import Tuple, List
-
-import py_cui.errors
+from typing import Tuple, List, Optional
 
 from qrogue.game.logic.actors import Player
 from qrogue.game.world.dungeon_generator import QrogueLevelGenerator, QrogueWorldGenerator
 from qrogue.game.world.map import CallbackPack
 from qrogue.game.world.navigation import Coordinate
-from qrogue.management import GameHandler, SaveData
-from qrogue.util import Config, Logger, RandomManager, PathConfig, GameplayConfig
+from qrogue.management import SaveData, QrogueCUI
+from qrogue.util import PyCuiConfig, Config, Logger, RandomManager, PathConfig, GameplayConfig
+from qrogue.util.key_logger import OverWorldKeyLogger
 
 
 def __init_singletons(seed: int):
@@ -41,20 +40,23 @@ def __init_singletons(seed: int):
     def open_riddle(robot, riddle):
         pass
 
+    def open_challenge(robot, challenge):
+        pass
+
     def visit_shop(robot, shop_item_list):
         pass
 
     def game_over():
         pass
 
-    CallbackPack(start_level, start_fight, start_boss_fight, open_riddle, visit_shop, game_over)
+    CallbackPack(start_level, start_fight, start_boss_fight, open_riddle, open_challenge, visit_shop, game_over)
 
 
-def __parse_argument(argument: List[str], has_value: bool = False) -> Tuple[bool, str]:
+def __parse_argument(argument: List[str], has_value: bool = False) -> Tuple[bool, Optional[str]]:
     for arg in argument:
         if arg in sys.argv:
             if has_value:
-                i = sys.argv.index(argument[0])
+                i = sys.argv.index(arg)
                 if i + 1 < len(sys.argv):
                     return True, sys.argv[i + 1]
             else:
@@ -62,6 +64,33 @@ def __parse_argument(argument: List[str], has_value: bool = False) -> Tuple[bool
     return False, None
 
 
+def start_qrogue() -> None:
+    __CONSOLE_ARGUMENT = ["--from-console", "-fc"]
+    __DEBUG_ARGUMENT = ["--debug", "-d"]
+    __TEST_LEVEL_ARGUMENT = ["--test-level", "-tl"]
+    __GAME_DATA_PATH_ARGUMENT = ["--game-data", "-gd"]  # path argument
+    __USER_DATA_PATH_ARGUMENT = ["--user-data", "-ud"]  # path argument
+    # __CONTROLS_ARGUMENT = ["--controls", "-c"]          # int argument
+    __SIMULATION_FILE_ARGUMENT = ["--simulation-path", "-sp"]   # path argument
+    __VALIDATE_MAP_ARGUMENT = ["--validate-map", "-vm"]     # path argument
+
+    from_console, _ = __parse_argument(__CONSOLE_ARGUMENT)
+    debugging, _ = __parse_argument(__DEBUG_ARGUMENT)
+    test_level, _ = __parse_argument(__TEST_LEVEL_ARGUMENT)
+    _, data_folder = __parse_argument(__GAME_DATA_PATH_ARGUMENT, has_value=True)
+    _, user_data_folder = __parse_argument(__USER_DATA_PATH_ARGUMENT, has_value=True)
+    _, simulation_path = __parse_argument(__SIMULATION_FILE_ARGUMENT, has_value=True)
+    _, map_path = __parse_argument(__VALIDATE_MAP_ARGUMENT, has_value=True)
+
+    if map_path:
+        validate_map(map_path)
+    else:
+        if simulation_path:
+            simulate_game(simulation_path, from_console, debugging, data_folder, user_data_folder)
+        else:
+            start_game(from_console, debugging, test_level, data_folder, user_data_folder)
+
+
 def setup_game(game_data_path: str = "", user_data_path: str = "") -> None:
     """
     Creates the needed folder structure and game config file qrogue_game.config if not already existent.
@@ -78,32 +107,21 @@ def setup_game(game_data_path: str = "", user_data_path: str = "") -> None:
     PathConfig.write(path, data, in_user_path=False, may_exist=True, append=False)
 
 
-def start_game(data_folder: str = None, user_data_folder: str = None):
-    __CONSOLE_ARGUMENT = ["--from-console", "-fc"]
-    __DEBUG_ARGUMENT = ["--debug", "-d"]
-    __GAME_DATA_PATH_ARGUMENT = ["--game-data", "-gd"]
-    __USER_DATA_PATH_ARGUMENT = ["--user-data", "-ud"]
-
-    exists, value = __parse_argument(__GAME_DATA_PATH_ARGUMENT, has_value=True)
-    if exists:
-        data_folder = value
-    exists, value = __parse_argument(__USER_DATA_PATH_ARGUMENT, has_value=True)
-    if exists:
-        user_data_folder = value
-
+def start_game(from_console: bool = False, debugging: bool = False, test_level: bool = False,
+               data_folder: str = None, user_data_folder: str = None):
     if PathConfig.load_paths(data_folder, user_data_folder):
         return_code = Config.load()  # NEEDS TO BE THE FIRST THING WE DO!
     else:
         return_code = 1
     if return_code == 0:
-        if __parse_argument(__DEBUG_ARGUMENT)[0]:
-            Config.activate_debugging()
+        if debugging:
+            Config.activate_debugging(test_level)
+
         seed = random.randint(0, Config.MAX_SEED)
         print(f"[Qrogue] Starting game with seed = {seed}")
         try:
-            game = GameHandler(seed)
-            game.start()
-        except py_cui.errors.PyCUIOutOfBoundsError:
+            QrogueCUI(seed).start()
+        except PyCuiConfig.OutOfBoundsError:
             #print("[Qrogue] ERROR!")
             #print("Your terminal window is too small. "
             #      "Please make it bigger (i.e. maximize it) or reduce the font size.")
@@ -112,20 +130,66 @@ def start_game(data_folder: str = None, user_data_folder: str = None):
 
         # flush after the player stopped playing
         if GameplayConfig.auto_save():
-            if SaveData.instance().save():
+            success, message = SaveData.instance().save()
+            if success:
                 print("[Qrogue] Successfully saved the game!")
             else:
-                Logger.instance().error("Failed to save the game.", show=False)
+                Logger.instance().error(f"Failed to save the game: {message.text}", show=False, from_pycui=False)
         Logger.instance().flush()
+        OverWorldKeyLogger.instance().flush_if_useful()
         print("[Qrogue] Successfully flushed all logs and shut down the game without any problems. See you next time!")
     else:
         print(f"[Qrogue] Error #{return_code}:")
         if return_code == 1:
             print("qrogue.config is invalid. Please check if the second line describes a valid path (the path "
-                  "to your save files). Using special characters in the path could also cause this error so if the path is "
-                  "valid please consider using another one without special characters.")
+                  "to your save files). Using special characters in the path could also cause this error so if the "
+                  "path is valid please consider using another one without special characters.")
 
-    if not __parse_argument(__CONSOLE_ARGUMENT)[0]:
+    if not from_console:
+        print()
+        input("[Qrogue] Press ENTER to close the application")
+
+    exit(return_code)
+
+
+def simulate_game(simulation_path: str, from_console: bool = False, debugging: bool = False, data_folder: str = None,
+                  user_data_folder: str = None):
+    if PathConfig.load_paths(data_folder, user_data_folder):
+        return_code = Config.load()  # NEEDS TO BE THE FIRST THING WE DO!
+    else:
+        return_code = 1
+    if return_code == 0:
+        if debugging:
+            Config.activate_debugging()
+
+        print(f"[Qrogue] Simulating the game recorded at \"{simulation_path}\"")
+        try:
+            QrogueCUI.start_simulation(simulation_path)
+        except PyCuiConfig.OutOfBoundsError:
+            # print("[Qrogue] ERROR!")
+            # print("Your terminal window is too small. "
+            #      "Please make it bigger (i.e. maximize it) or reduce the font size.")
+            print("---------------------------------------------------------")
+            exit(1)
+
+        # flush after the player stopped playing
+        if GameplayConfig.auto_save():
+            success, message = SaveData.instance().save()
+            if success:
+                print("[Qrogue] Successfully saved the game!")
+            else:
+                Logger.instance().error(f"Failed to save the game: {message.text}", show=False, from_pycui=False)
+        Logger.instance().flush()
+        OverWorldKeyLogger.instance().flush_if_useful()
+        print("[Qrogue] Successfully flushed all logs and shut down the game without any problems. See you next time!")
+    else:
+        print(f"[Qrogue] Error #{return_code}:")
+        if return_code == 1:
+            print("qrogue.config is invalid. Please check if the second line describes a valid path (the path "
+                  "to your save files). Using special characters in the path could also cause this error so if the "
+                  "path is valid please consider using another one without special characters.")
+
+    if not from_console:
         print()
         input("[Qrogue] Press ENTER to close the application")
 
@@ -151,11 +215,14 @@ def validate_map(path: str, is_level: bool = True, in_base_path: bool = True) ->
     def load_map(map_name: str, spawn_pos: Coordinate):
         pass
 
+    def show_message(title: str, text: str):
+        pass
+
     if is_level:
-        generator = QrogueLevelGenerator(seed, check_achievement, trigger_event, load_map)
+        generator = QrogueLevelGenerator(seed, check_achievement, trigger_event, load_map, show_message)
     else:
         player = Player()
-        generator = QrogueWorldGenerator(seed, player, check_achievement, trigger_event, load_map)
+        generator = QrogueWorldGenerator(seed, player, check_achievement, trigger_event, load_map, show_message)
 
     try:
         error_occurred = False
