@@ -1,6 +1,5 @@
 # exporting
-from .config import CheatConfig, ColorConfig, GameplayConfig, InstructionConfig, MapConfig, PathConfig, PopupConfig, \
-    Config, ShopConfig
+from .config import *
 from .controls import Controls, Keys
 from .help_texts import HelpText, HelpTextType
 from .logger import Logger
